"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _awsSdk = require("aws-sdk");

var _bluebird = _interopRequireDefault(require("bluebird"));

var _retry = require("../utils/retry");

/* eslint import/no-extraneous-dependencies: ["error", {"devDependencies": true}] */
_awsSdk.config.setPromisesDependency(_bluebird.default);

class Connector {
  constructor({
    debug,
    streamName = process.env.STREAM_NAME,
    timeout = Number(process.env.KINESIS_TIMEOUT) || Number(process.env.TIMEOUT) || 1000,
    retryConfig = _retry.defaultRetryConfig
  }) {
    this.debug = msg => debug('%j', msg);

    this.streamName = streamName || 'undefined';
    this.stream = new _awsSdk.Kinesis({
      httpOptions: {
        timeout,
        connectTimeout: timeout
      },
      maxRetries: 10,
      // Default: 3
      retryDelayOptions: {
        base: 200
      },
      // Default: 100 ms
      logger: {
        log:
        /* istanbul ignore next */
        msg => debug('%s', msg.replace(/\n/g, '\r'))
      }
    });
    this.retryConfig = retryConfig;
  }

  putRecords(inputParams) {
    const params = {
      StreamName: this.streamName,
      ...inputParams
    };
    return this._putRecords(params, []);
  }

  _putRecords(params, attempts) {
    (0, _retry.assertMaxRetries)(attempts, this.retryConfig.maxRetries);
    return (0, _retry.wait)((0, _retry.getDelay)(this.retryConfig.retryWait, attempts.length)).then(() => this.stream.putRecords(params).promise().tap(this.debug).tapCatch(this.debug).then(resp => {
      if (resp.FailedRecordCount > 0) {
        return this._putRecords(unprocessed(params, resp), [...attempts, resp]);
      } else {
        return accumlate(attempts, resp);
      }
    }));
  }

}

var _default = Connector;
exports.default = _default;

const unprocessed = (params, resp) => ({ ...params,
  Records: params.Records.filter((e, i) => resp.Records[i].ErrorCode)
});

const accumlate = (attempts, resp) => attempts.reduceRight((a, c) => ({ ...a,
  Records: [...c.Records.filter(r => !r.ErrorCode), ...a.Records.filter(r => !r.ErrorCode)],
  attempts: [...attempts, resp]
}), resp);