"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _awsSdk = require("aws-sdk");

var _bluebird = _interopRequireDefault(require("bluebird"));

var _retry = require("../utils/retry");

/* eslint import/no-extraneous-dependencies: ["error", {"devDependencies": true}] */
_awsSdk.config.setPromisesDependency(_bluebird.default);

class Connector {
  constructor({
    debug,
    queueUrl = process.env.QUEUE_URL,
    timeout = Number(process.env.SQS_TIMEOUT) || Number(process.env.TIMEOUT) || 1000,
    retryConfig = _retry.defaultRetryConfig
  }) {
    this.debug = msg => debug('%j', msg);

    this.queueUrl = queueUrl || 'undefined';
    this.queue = new _awsSdk.SQS({
      httpOptions: {
        timeout,
        connectTimeout: timeout
      },
      maxRetries: 10,
      // Default: 3
      retryDelayOptions: {
        base: 200
      },
      // Default: 100 ms
      logger: {
        log:
        /* istanbul ignore next */
        msg => debug('%s', msg.replace(/\n/g, '\r'))
      }
    });
    this.retryConfig = retryConfig;
  }

  sendMessageBatch(inputParams) {
    const params = {
      QueueUrl: this.queueUrl,
      ...inputParams
    };
    return this._sendMessageBatch(params, []);
  }

  _sendMessageBatch(params, attempts) {
    (0, _retry.assertMaxRetries)(attempts, this.retryConfig.maxRetries);
    return (0, _retry.wait)((0, _retry.getDelay)(this.retryConfig.retryWait, attempts.length)).then(() => this.queue.sendMessageBatch(params).promise().tap(this.debug).tapCatch(this.debug).then(resp => {
      var _resp$Failed;

      if (((_resp$Failed = resp.Failed) === null || _resp$Failed === void 0 ? void 0 : _resp$Failed.length) > 0) {
        return this._sendMessageBatch(unprocessed(params, resp), [...attempts, resp]);
      } else {
        return accumlate(attempts, resp);
      }
    }));
  }

}

var _default = Connector;
exports.default = _default;

const unprocessed = (params, resp) => ({ ...params,
  Entries: resp.Failed.map(m => params.Entries.find(e => m.Id === e.Id))
});

const accumlate = (attempts, resp) => attempts.reduceRight((a, c) => ({ ...a,
  Successful: [...c.Successful, ...a.Successful],
  attempts: [...attempts, resp]
}), resp);