"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.fetch = void 0;

var _highland = _interopRequireDefault(require("highland"));

var _fetch = _interopRequireDefault(require("../connectors/fetch"));

var _faults = require("./faults");

var _print = require("./print");

var _ratelimit = require("./ratelimit");

const fetch = ({
  // eslint-disable-line import/prefer-default-export
  debug = (0, _print.debug)('fetch'),
  prefix = 'fetch',
  httpsAgent,
  parallel = Number(process.env.FETCH_PARALLEL) || Number(process.env.PARALLEL) || 8,
  ...opt
} = {}) => {
  const connector = new _fetch.default({
    debug,
    httpsAgent
  });

  const invoke = uow => {
    const {
      url,
      responseType = 'json',
      ...request
    } = uow[`${prefix}Request`];
    const p = connector.fetch(url, request, responseType).then(response => ({ ...uow,
      [`${prefix}Response`]: response
    })).catch((0, _faults.rejectWithFault)(uow));
    return (0, _highland.default)(p); // wrap promise in a stream
  };

  return s => s.through((0, _ratelimit.ratelimit)(opt)).map(invoke).parallel(parallel);
};

exports.fetch = fetch;